import os
import sys

import EMAN2



def readEMAN1ctf():
	source_ctf = {}
	
	try:
		f = open("ctfparm.txt", "r")
		r = [i.strip() for i in f.readlines()]
		f.close()
	except:
		print "No ctfparm.txt present; could not load EMAN1 CTF parameters"
		return

	indexes = {
		"ctf_defocus_measured": 0, # multiply by -1
		"ctf_bfactor": 3,
		"tem_voltage": 10,
		"aberration_spherical": 11,
		"angstroms_per_pixel": 12,
		"ctf_astig_angle": 2 ,
		"ctf_astig_defocus_diff": 1,
		"ctf_ampcont": 5, # multiply by 100
	}

	for i in r:
		try:
			source, params = i.split("\t")
			params = params.split(",")

			# it seems that if there are 14 params, astig params are inserted as 1,2
			shift = 0
			if len(params) == 14:
				shift = 2

			ctf = EMAN2.EMAN2Ctf()
			ctf.defocus = float(params[0]) * -1
			ctf.bfactor = float(params[1+shift]) * 4 # Convert to Henderson convention
			ctf.ampcont = float(params[3+shift]) * 100

			# print "defocus %s, bfactor %s, ampcont %s"%(ctf.defocus, ctf.bfactor, ctf.ampcont)
			source_ctf[source] = ctf
		except:
			print "Unable to parse CTF parameters, skipping"
			
	return source_ctf
	
	
	
	
def readEMAN1boxes():
	source_box_size = {}
	source_boxes = {}
	boxes = glob.glob("*.box")

	if not boxes:
		print "No EMAN1 .box files found"
		return

	print "Found EMAN1 boxes: %s"%boxes

	for box in boxes:
		try:
			source = os.path.splitext(box)[0]
			box_size, coords = readEMAN1box(box)
		except:
			print "Could not load data from box %s, skipping"%box
			continue

		source_box_size[source] = box_size
		source_boxes[source] = coords	
	
	return source_box_size, source_boxes		
			
			
			
			
def readEMAN1box(box, shrink_factor=1.0):
	f = open(box, "r")
	r = [i.split() for i in f.readlines()]
	f.close()

	coords = []
	for b in r:
		box_size = int(b[2])
		xc = (int(b[0]) + (box_size/2)) / shrink_factor
		yc = (int(b[1]) + (box_size/2)) / shrink_factor
		coords.append([int(xc),int(yc)])

	box_size = int(box_size / shrink_factor)

	return box_size, coords			




def readEMAN2(self):

	###############
	# Find all the raw images, particle sets, and CTF parameters in the local DB
	###############

	projdb = EMAN2.db_open_dict("bdb:project")
	ptclsets = projdb.get("global.spr_ptcls_dict", {})
	e2ctfparms = EMAN2.db_open_dict("bdb:e2ctf.parms")
	total = len(ptclsets)

	# Read the EMAN2 managed particle sets
	for count, (k,v) in enumerate(ptclsets.items()):
		ref = v.get('Phase flipped') or v.get('Original Data')
		if not ref:
			print "No particles found for %s, skipping"%k
			continue

		print "%s of %s: Getting info for particle set %s from %s"%(count+1, total, k, ref)

		d = EMAN2.db_open_dict(ref)

		coords = []
		maxrec = d['maxrec']
		if maxrec == None:
			print "No particles in %s, skipping"%(k)
			d.close()
			continue


		# Get info from first particle in stack
		ptcl = d[0]
		try: ctf = ptcl.get_attr("ctf")
		except: ctf = None

		try: box_size = int(ptcl.get_attr("nx") / self.options.shrink_factor)
		except: box_size = None

		# Mangle source name if necessary
		try:
			source = os.path.basename(ptcl.get_attr('ptcl_source_image'))
		except:
			source = k
			source = source.split("_ptcl")[0]
			print "\tUnable to get source image %s, using %s for the filename search"%(k,k)


		# Try to read boxes from particle headers
		if self.options.boxes:
			for i in range(maxrec+1):
				dptcl = d[i]
				try:
					x, y = dptcl.get_attr('ptcl_source_coord')
					x /= self.options.shrink_factor
					y /= self.options.shrink_factor
					coords.append([int(x), int(y)])
				except:
					coords.append(None)


			if None in coords:
				print "\tSome particles for %s did not specify coordinates"%k
				coords = []

			print "Got box_size %s and coords %s"%(box_size, coords)



		# Get alternative CTF and quality from e2ctfit settings
		ctf2, quality = readEMAN2e2ctfparms(e2ctfparms, k)
		if not ctf and ctf2:
			print "\tUsing CTF parameters from bdb:e2ctf.parms#%s"%k
			ctf = ctf2

		if ctf:
			self.source_ctf[source] = ctf
			print "Got CTF: defocus %s, B-factor %s"%(ctf.defocus, ctf.bfactor)
		else:
			print "\tNo CTF for %s"%k

		if box_size and coords:
			self.source_box_size[source] = box_size
			self.source_boxes[source] = coords

		if quality:
			self.source_quality[source] = quality

		d.close()


	# If we can't find any EMAN2 managed particle sets, at least check e2ctf.parms for any CTF params
	if not ptclsets:
		print "No EMAN2 managed particle sets found; checking e2ctf.parms for CTF instead"

		for k,v in e2ctfparms.items():
			ctf, quality = readEMAN2e2ctfparms(e2ctfparms, k)
			source = k.split("_ptcl")[0]

			if ctf:
				self.source_ctf[source] = ctf
			if quality:
				self.source_quality[source] = quality

	print "\n%s Files in local project: "%len(self.source_ctf), self.source_ctf.keys()

	projdb.close()
	e2ctfparms.close()



def readEMAN2e2ctfparms(self, e2ctfparms, item):
	ctf2 = None
	quality = None

	ctf_str = e2ctfparms.get(item)
	if ctf_str:
		try:
			ctf2 = EMAN2.EMAN2Ctf()
			ctf2.from_string(ctf_str[0])
			quality = ctf_str[-1]
		except:
			print "\tProblem reading CTF from bdb:e2ctf.parms for %s"%item

	return ctf2, quality


def filematch(i, j):
	if i.lower() in j.lower(): return True



# def getremoteinfo(self):
# 	# Since filenames are not guaranteed unique, restrict to a project...
# 	print "\nChecking project %s on %s"%(self.project, self.db._host)
# 
# 	self.projrecs = self.db.getchildren(self.project, -1, ["ccd","scan"])
# 	print "\tFound %s ccds/scans in remote project"%len(self.projrecs)
# 
# 	self.bdos = self.db.getbinary(self.projrecs)
# 	print "\tFound %s binaries"%len(self.bdos)



def findbinary(sources, project, clip_filename=None, db=None):
	###############
	# For each file in the local project, search for a matching BDO and record ID in the database
	###############
	
	# Since filenames are not guaranteed unique, restrict to a project...
	print "\nChecking project %s"%(project)

	projrecs = db.getchildren(project, -1, ["image_capture*"])
	print "\tFound %s images in remote project"%len(projrecs)

	bdos = db.getbinary(projrecs)
	print "\tFound %s binaries"%len(bdos)
	
	total = len(sources)
	source_bdo = {}

	bdosbyfilename = dict([[i["filename"], i] for i in bdos])
	filenames = bdosbyfilename.keys()

	ambiguous = {}
	nomatches = {}

	# ian: new style: I fixed getbinary, so just do one big BDO lookup instead of many db.findbinary

	print "remote filenames?:"
	print filenames

	for count, source in enumerate(sources):
		print "\n%s of %s: Looking up file %s in project %s"%(count+1, total, source, project)

		q = source.replace(clip_filename or '', "")
		gzipstrip = ".gz" in source

		# ian: replace this with functools.partial
		# matches = map(bdosbyfilename.get, filter(lambda x:q in x.split("."), filenames))
		matches = []
		for i in filenames:
			i2 = i
			if gzipstrip:
				i2 = i.replace('.gz','')
			#if q in i2:
			if filematch(q, i2):
				matches.append(bdosbyfilename.get(i))


		if len(matches) > 1:
			print "\tAmbiguous match for %s: %s"%(source, [match["filename"] for match in matches])
			ambiguous[source] = matches
			continue


		if len(matches) == 0:
			print "\tNo matches for %s"%source
			nomatches[source] = []
			continue


		match = matches[0]
		print "\tFound %s in record %s, matching filename is %s"%(match["name"], match["recid"], match["filename"])

		source_bdo[source] = match


	print "\n\nSuccessful Local-Remote Mappings:"
	for k,v in source_bdo.items():
		print "\t%s -> %s (recid %s)"%(k, v["filename"], v["recid"])

	if ambiguous:
		print "\n\nAmbiguous matches:"
		for k,v in ambiguous.items():
			print "\t%s -> %s"%(k, v)

	if nomatches:
		print "\n\nNo matches:"
		for k in nomatches:
			print "\t%s"%k

	return source_bdo
			