# $Id: emen2proxy.py,v 1.2 2010/12/09 21:59:41 edwlan Exp $
from .jsonrpc import proxy
import emen2.db.paramdef
import emen2.db.recorddef
import emen2.db.record
import emen2.db.user
import copy

def zipdicts(*dicts):
	keydict = dicts[0]
	for k in keydict.iterkeys():
		yield (k, tuple(dict_.get(k) for dict_ in dicts))

def newrecord(rec):
	r = emen2.db.record.Record(rec)
	return r

def newthing(typ):
	def _inner(data):
		a = object.__new__(typ)
		a.update(data)
		return a
	return _inner

class Context(object):
	def __init__(self, ctxid, host):
		self.ctxid = ctxid
		self.host = host
		self.db = None


class Emen2Proxy(proxy.JSONRPCProxy):
	def init(self, ctxid=None, t=False, *args, **kwargs):
		self._ctx = Context(ctxid, self.serviceURL)
		self._ctx.db = self
		self.__convert = True
		self.__t = t

	def transformURL(self, url, path):
		if self.__t:
			path = '/?'.join([path, 't=1'])
		return url, path

	typemapping = dict(
		set=set,
		tuple=tuple,
		record=newrecord,
		paramdef=newthing(emen2.db.paramdef.ParamDef),
		recorddef=newthing(emen2.db.recorddef.RecordDef),
		user=newthing(emen2.db.user.User)
	)
	revmap = dict( (y,x) for x,y in typemapping.iteritems() )

	def restore_from_typemap(self, typemap, val):
		result = val
		containertype = None
		if isinstance(typemap, list):
			result = {}
			for k, (vv, tv) in zipdicts(val, typemap[1]):
				result[k] = self.restore_from_typemap(tv, vv)
			containertype=self.typemapping.get(typemap[0])
		else:
			containertype=self.typemapping.get(typemap)
		if containertype is not None:
			result = containertype(result)
		return result

	def __getattr__(self, name):
		result = emen2.clients.jsonrpc.proxy.JSONRPCProxy.__getattr__(self, name)
		result.set_ctxid(self._ctx.ctxid)
		return result

	def get_ctxid(self):
		return self._ctx.ctxid
	def set_ctxid(self, ctxid):
		self._ctx.ctxid = ctxid

	def get_postdata(self, args, kwargs):
		if self._ctx.ctxid: kwargs['ctxid'] = self._ctx.ctxid
		return emen2.clients.jsonrpc.proxy.JSONRPCProxy.get_postdata(self, args, kwargs)

	def proc_response(self, data):
		result = copy.deepcopy(data['result'])
		typemap = copy.deepcopy(data.get('type'))
		if typemap:
			result = self.restore_from_typemap(typemap, result)
		result = dict(
			result = result
		)
		return result

	def _login(self, username, password):
		self._ctx.ctxid = self.login(username, password)

__version__ = "$Revision: 1.2 $".split(":")[1][:-1].strip()
