# $Id: p41.py,v 1.30 2010/10/28 05:05:56 irees Exp $
import cStringIO
import collections

import emen2.util.listops
import emen2.db.config
g = emen2.db.config.g()
from emen2.web.view import View

def sort_grant_source(pub):
	if pub["grant_source"] == "NIH": return 2
	if pub["grant_source_type"] == "USA_Federal": return 1
	return 0



@View.register
class P41Report(View):

	@View.add_matcher(r'^/record/(?P<recid>\d+)/p41report/$', r'^/record/(?P<recid>\d+)/p41report/(?P<action>.+)/$')
	def init(self, recid=None, action=None, **kwargs):

		self.template = '/pages/p41'
		if action == 'years':
			self.template = '/pages/p41.years'
		elif action == 'xml':
			self.template = '/pages/p41.xml'
			self.mimetype = 'text/xml'
		

		# Get the p41_grant_info
		root = self.db.getrecord(recid)
		year = root.get('p41_year')
		print "Building report for %s"%year
		
		
		# Get the p41 projects
		recids = set()
		p41_recids = self.db.getchildren(recid, rectype="p41_project")
		p41_projects = self.db.getrecord(p41_recids)
		print "Total p41 projects for %s: %s"%(year, len(p41_projects))

		
		# Referenced users
		users = set(root['grant_pi'])
		usersbyproject = {}
		for rec in p41_projects:
			u = set([rec['p41_project_pi']] + rec.get('p41_project_investigators', []))
			if len(u) == 0:
				print "WARNING! Project %s has no investigators"%rec.recid
			usersbyproject[rec.recid] = u
			users |= u
				
				
		# Store this as a dict to make referencing easier
		users = self.db.getuser(users, lnf=1)
		users = emen2.util.listops.dictbykey(users, 'username')
		print "Total users for %s: %s"%(year, len(users))
		
		
		# Grants are children of a user, and parents of a p41 project
		grantsbyuser = collections.defaultdict(list)
		projectsbygrant = collections.defaultdict(set)
		grants = set()
		for user in users.values():
			print "Grants for user %s:"%(user.username)
			g = self.db.getrecord(self.db.getchildren(user.record, rectype="grant"))
			g_expired = filter(lambda x:int(x['grant_p41_ref_end'][:4]) < year, g)
			g = filter(lambda x:int(x['grant_p41_ref_end'][:4]) >= year, g)
			print "\tCurrent: %s, Expired: %s"%(len(g), len(g_expired))
			
			for grant in g:
				print "\tGrant %s"%grant.recid
				v = self.db.getchildren(grant.recid, rectype='p41_project') & p41_recids
				if len(v) == 0:
					print "\t\tDoes not reference any current projects..."

				for v2 in v:
					if user.username not in usersbyproject[v2]:
						print "\t\tUser is not referneced in project %s"%v2
					else:
						print "\t\tReferences: %s"%v2
						projectsbygrant[grant.recid].add(v2)
						grants.add(grant.recid)
						if grant.recid not in [j.recid for j in grantsbyuser[user.username]]:
							grantsbyuser[user.username].append(grant)
			
			grantsbyuser[user.username] = sorted(grantsbyuser[user.username], key=sort_grant_source)


		print "Total included grants for %s: %s"%(year, len(grants))
		
		
		# Back up each p41 project to find the associated publications.
		pubsbyproject = {}
		pubs = set()
		parents = self.db.getparents([i.recid for i in p41_projects], rectype="project*")
		for k, v in parents.items():
			print "Finding publications for %s"%k
			p = self.db.getchildren(v, rectype="publication*", recurse=-1)			
			if p.values():
				p = emen2.util.listops.combine(*p.values())
			p = self.db.getrecord(p) or []
			p_expired = filter(lambda x:x.get('p41_year')!=year, p)
			p = filter(lambda x:x.get('p41_year')==year and x.get('p41_acknowledged'), p)
			print "\tCurrent: %s, Other years: %s"%(len(p), len(p_expired))
			p = set([i.recid for i in p])
			pubsbyproject[k] = p
			pubs |= p
		
		
		recnames = self.db.renderview(p41_recids | grants | pubs, viewtype="recname")
		rendered = self.db.renderview(p41_recids | grants, viewtype="defaultview", mode="htmledit")
		# rendered.update(self.db.renderview(pubs, viewtype="recname"))
		
		pubs = self.db.getrecord(pubs)
		grants = self.db.getrecord(grants)
		print "Total publications for %s: %s"%(year, len(pubs))
			
		# Used by all templates
		self.ctxt['root'] = root
		self.ctxt['year'] = year
		self.ctxt['users'] = users
		self.ctxt['p41_projects'] = p41_projects
		self.ctxt['pubs'] = pubs
		self.ctxt['grants'] = grants
		self.ctxt['grantsbyuser'] = grantsbyuser
		self.ctxt['projectsbygrant'] = projectsbygrant
		self.ctxt['pubsbyproject'] = pubsbyproject

		# These are used by the editing interface
		self.ctxt['usersbyproject'] = usersbyproject
		self.ctxt['recnames'] = recnames
		self.ctxt['rendered'] = rendered
		



__version__ = "$Revision: 1.30 $".split(":")[1][:-1].strip()
