# $Id: record.py,v 1.177 2010/12/09 09:30:52 irees Exp $
import urllib

# Standard View imports
import emen2.db.config
g = emen2.db.config.g()
from emen2.web.view import View


import emen2.web.views.map
import emen2.web.markuputils
import emen2.util.listops as listops
import emen2.web.responsecodes



class RecordNotFoundError(emen2.web.responsecodes.NotFoundError):
	title = 'Record not Found'
	msg = 'Record %s not found'



class RecordBase(View):
	def _init(self, recid=None, children=True, parents=True, notify=None):
		"""Main record rendering."""
		
		# rec/relationships
		#try:
		self.recid = int(recid)
		# except ValueError:
		# 	page = urllib.unquote_plus(recid)
		# 	recids = self.db.getindexbyvalue("name_folder", page)
		# 	if len(recids) > 1:
		# 		raise RecordNotFoundError, "Disambiguation for %s: %s"%(page, recids)
		# 		#self.error()
		# 		
		# 	if len(recids) == 0:
		# 		raise RecordNotFoundError, "No matches for page %s"%page
		# 		# self.error("No matches for page %s"%page)
		# 	
		# 	self.recid = recids.pop()

		try:
			self.rec = self.db.getrecord(self.recid, filt=False)
		except (KeyError, TypeError), inst:
			raise RecordNotFoundError, recid
		#self.error("Could not access record %s"%recid)
		

		self.recdef = self.db.getrecorddef(self.rec.rectype)
		renderedrecname = self.db.renderview(self.rec, viewtype="recname") or "%s: %s"%(self.rec.rectype, self.rec.recid)

		# User display names
		# These are generally displayed: creator, modifyuser, comments. Others will be retrieved when they are needed, generally.
		getusers = set([self.rec.get('creator'), self.rec.get('modifyuser')])
		getusers |= set([i[0] for i in self.rec.get('comments',[])])
		getusers = self.db.getuser(getusers)
		displaynames = {}
		for user in getusers:
			displaynames[user.username] = user.displayname


		# Parent Map
		if parents:
			parentmap = emen2.web.views.map.Map(mode="parents", keytype="record", root=self.recid, db=self.db)
			parentmap_ctxt = parentmap.get_context()
			# renderedrecname = parentmap_ctxt.get("recnames",{}).get(self.recid,self.recid)
			parents = parentmap_ctxt.get("results",{}).get(self.recid,[])

			pages_map = {
				'classname':'map',
				'content':{'parents':parentmap, 'children': 'Loading...'},
				'active':'parents',
				'order':['parents','children'],
				'labels':{'parents':'Parents','children':'Children'}
			}
			pages_map = emen2.web.markuputils.HTMLTab(pages_map)

		else:
			pages_map = None


		# Child pages
		if children:			
			children = self.db.getchildren(self.recid)
			self.childgroups = self.db.groupbyrecorddef(children)

			pages = {
				'classname':'main',
				'content':{},
				'labels':{'main':'<span data-recid="%s" data-viewtype="recname" class="view">%s</span>'%(self.recid, renderedrecname)},
				'href':	{'main':'%s/record/%s/'%(g.EMEN2WEBROOT,self.recid)},
				'active': 'main',
				'order': ['main']
			}

			for k,v in self.childgroups.items():
				pages["order"].append(k)
				pages["labels"][k] = "%s (%s)"%(k,len(v))
				pages["content"][k] = ""
				pages["href"][k] = self.dbtree.reverse('Record/children', recid=self.recid, childtype=k)
				
			pages = emen2.web.markuputils.HTMLTab(pages)
		else:
			pages = None


		# Update context
		self.update_context(
			rec = self.rec,
			recs = {str(self.recid):self.rec},
			recdef = self.recdef,
			notify = notify or [],
			title = "Record: %s: %s (%s)"%(self.rec.rectype, renderedrecname, self.recid),
			renderedrecname = renderedrecname,
			displaynames = displaynames,
			pages = pages,
			pages_map = pages_map,
			viewtype = "defaultview",
			edit = False,
			key = self.recid,
			maptype = "record",
			create = self.db.checkcreate()
		)





@View.register
class Record(RecordBase):
	
	@View.add_matcher(r'^/record/(?P<recid>\w+)/$')
	def main(self, recid=None, sibling=None, notify=None):
		self._init(recid=recid, notify=notify)
		self.template = '/pages/record.main'
		
		# Siblings
		if sibling == None:
			sibling = self.rec.recid
		sibling = int(sibling)
		siblings = self.db.getsiblings(sibling, rectype=self.rec.rectype)

		viewtype = "defaultview"
		if not self.recdef.views.get("defaultview"):
			viewtype = "mainview"

		mode = "html"
		if self.rec.writable():
			mode = "htmledit"

		# Render main view
		rendered = self.db.renderview(self.rec, viewtype=viewtype, mode=mode)

		historycount = len(self.rec.get('history',[]))
		historycount += len(filter(lambda x:x[2].startswith("LOG:"), self.rec.get('comments',[])))

		pages_comments = {
			'classname':"comments",
			'switched':1,
			'content':{"comments":"","history":"Loading..."},
			'labels':{"comments":"Comments","history":"History (%s changes)"%(historycount)},
			'order':["comments","history"],
		}
		pages_comments = emen2.web.markuputils.HTMLTab(pages_comments)

		#######################################

		self.update_context(
			viewtype = viewtype,
			rendered = rendered,
			sibling = sibling,
			siblings = sorted(siblings),
			pages_comments = pages_comments,
		)

	
	
	@View.add_matcher(r'^/record/(?P<recid>\d+)/edit/$')
	def edit(self, recid=None, notify=None):	
		self.main(recid=recid, notify=notify)
		self.ctxt["edit"] = True
	
	
	

	@View.add_matcher(r'^/record/(?P<recid>\d+)/new/(?P<rectype>\w+)/$')
	def new(self, recid=None, rectype=None, inheritperms=None, copy=None, viewtype='mainview', notify=None):
		self._init(recid=recid, notify=notify, children=False)

		self.template = '/pages/record.new'
		self.title = "New Record"

		if not self.db.checkcreate():
			raise emen2.db.exceptions.SecurityError, "No permission to create records"
				
				
		if inheritperms == None:
			inheritperms = self.recid
		else:
			try:
				inheritperms = int(inheritperms)
			except:
				inheritperms = None
			
		user = self.db.checkcontext()[0]

		newrec = self.db.newrecord(rectype, init=False, inheritperms=inheritperms)
		newrec["parents"] = [self.recid]

		if copy:
			for k in self.rec.getparamkeys():
				newrec[k] = self.rec[k]


		rendered = self.db.renderview(newrec, mode="htmledit", viewtype=viewtype)
		recdef = self.db.getrecorddef(newrec.rectype)

		recorddefnames = self.db.getrecorddefnames()

		self.update_context(
			recdef = recdef,
			newrec = newrec,
			viewtype = viewtype,
			rendered = rendered,
			recorddefnames = recorddefnames
		)



	@View.add_matcher('^/record/(?P<recid>\d+)/children/(?P<childtype>\w+)/$')
	def children(self,recid=None,childtype=None,notify=None):
		"""Main record rendering."""		
		self._init(recid=recid, notify=notify)
		self.template = "/pages/record.table"
		self.ctxt["create"] = self.db.checkcreate()
		self.ctxt["childtype"] = childtype
		c = [['children', 'recid', recid], ['rectype', '==', childtype]]
		self.ctxt["q"] = self.db.querytable(c=c)
		self.ctxt["pages"].setactive(childtype)



	#@write
	@View.add_matcher("^/record/(?P<recid>\d+)/delete/$")
	def delete(self, commit=False, recid=None, notify=None):
		"""Main record rendering."""

		self._init(recid=recid, notify=notify)
		self.template = "/pages/record.delete"
		self.title = "Delete Record"

		orphans = checkorphans(self.db, self.recid)
		recnames = self.db.renderview(orphans, viewtype='recname')

		if commit:
			self.db.deleterecord(self.recid)

		self.ctxt["commit"] = commit
		self.ctxt["orphans"] = orphans
		self.ctxt["recnames"] = recnames



	@View.add_matcher("^/record/(?P<recid>\d+)/history/$", "^/record/(?P<recid>\d+)/history/(?P<revision>.*)/")
	def history(self, recid=None, notify=None, simple=False, revision=None):
		"""Revision/history/comment viewer"""

		if revision:
			revision = revision.replace("+", " ")

		if simple:
			self._init(recid=recid, notify=notify, parents=False, children=False)
			self.template = "/pages/record.history"
		else:
			self._init(recid=recid, notify=notify, children=False)
			self.template = "/pages/record.revision"

		rendered = ""
		if revision:
			try:
				revs, p = self.rec.revision(revision)
				self.rec.update(p)
				rendered = self.db.renderview(self.rec, mode="html", viewtype="mainview")
			except:
				rendered = "Could not render for this revision -- parameter definitions may have changed since."

		self.title = "History"
		self.ctxt["revision"] = revision
		self.ctxt["rendered"] = rendered
		



	@View.add_matcher("^/record/(?P<recid>\d+)/email/$")
	def email(self, recid=None, notify=None):
		"""Main record rendering."""

		self._init(recid=recid, notify=notify)
		self.template = "/pages/record.email"
		self.title = "Users referenced by record %s"%(self.recid)

		# ian: todo: replace!!
		pds = self.db.getparamdef(self.rec.keys())
		users_ref = set()
		for i in pds:
			v = self.rec.get(i.name)
			if not v:
				continue
			if i.vartype == "user":
				users_ref.add(v)
			elif i.vartype == "userlist":
				users_ref |= set(v)

		users_permissions = set()
		for v in self.rec.get('permissions'):
			users_permissions |= set(v)
		
		users = listops.dictbykey(self.db.getuser(users_ref | users_permissions, filt=True, lnf=True), 'username')

		self.update_context(
			users = users,
			users_ref = users_ref,
			users_permissions = users_permissions,
		)



	@View.add_matcher(r'^/record/(?P<recid>\d+)/publish/$')
	def publish(self, recid=None):
		self._init(recid=recid)
		self.template = '/pages/record.publish'
		self.title = 'Publish Records'
		
		status = self.db.getindexbypermissions(groups=["publish"])
		childmap_view = emen2.web.views.map.Map(mode="children", keytype="record", root=self.recid, db=self.db)
		childmap = childmap_view.get_data()

		# print status
		
		self.set_context_item("childmap", childmap)
		self.set_context_item("collapsed", childmap_view.collapsed)
		self.set_context_item("children", childmap_view.collapsedchildren)
		self.set_context_item("status", status)




	@View.add_matcher(r'^/record/(?P<recid>\d+)/boxer/$')
	def boxer(self, recid=None):
		self._init(recid=recid)
		self.template = '/pages/boxer'
		self.title = "web.boxer (EXPERIMENTAL!)"

		boxrecords = self.db.getchildren(self.rec.recid, rectype="box")
		bdo = self.rec.get('file_binary_image')
		bdo = self.db.getbinary(bdo)
		if not bdo:
			self.error("No ccd frames found! Cannot start web.boxer.")
			return

		self.ctxt["bdo"] = bdo


		
	# @View.add_matcher(r'^/wiki/(?P<page>.+)/edit/$')
	# def edit(self, page=None):
	# 	self._init(recid=page, children=False, parents=False)
	# 	self.template = '/pages/wiki.edit'
	# 	self.ctxt["rendered"] = self.db.renderview(self.rec, viewtype="mainview", mode="htmledit")
		

	# @View.add_matcher(r'^/record/view/(?P<recid>.+)/(?P<viewtype>\w+)/')
	# def view(self, recid=None, viewtype="defaultview", notify=None):
	# 	self.template = '/basedb/raw'
	# 	self.ctxt["content"] = self.db.renderview(self.rec, viewtype=viewtype, mode=mode)




# moved from db...
def checkorphans(db, recid):
	"""Find orphaned records that would occur if recid was deleted.

	@param recid Return orphans that would result from deletion of this Record

	@return Set of orphaned Record IDs
	"""
	srecid = set([recid])
	saved = set()

	# this is hard to calculate
	children = db.getchildtree(recid, recurse=-1)
	orphaned = reduce(set.union, children.values(), set())
	orphaned.add(recid)
	parents = db.getparenttree(orphaned)

	# orphaned is records that will be orphaned if they are not rescued
	# find subtrees that will be rescued by links to other places
	for child in orphaned - srecid:
		if parents.get(child, set()) - orphaned:
			saved.add(child)

	children_saved = db.getchildtree(saved, recurse=-1)
	children_saved_set = set()
	for i in children_saved.values() + [set(children_saved.keys())]:
		children_saved_set |= i
	# .union( *(children_saved.values()+[set(children_saved.keys())], set()) )

	orphaned -= children_saved_set

	return orphaned
		
		
		

__version__ = "$Revision: 1.177 $".split(":")[1][:-1].strip()
